#ifndef _W32LIB_H_
#define _W32LIB_H_

#include <windows.h>
#include <io.h>
#include <stdio.h>
#include <glib.h>
#ifdef __MINGW32__
#include <wchar.h>
#endif

// * types *
// *** ??? ***
#ifndef __MINGW32__
typedef long int off_t;
typedef int pid_t;
typedef unsigned char u_char;
#endif /* __MINGW32__ */
typedef unsigned int uid_t;

#ifndef __MINGW32__
// *** stat ***
#define S_IRUSR	_S_IREAD
#define S_IWUSR	_S_IWRITE
#define S_IXUSR	_S_IEXEC
#define S_IRWXU	(_S_IREAD|_S_IWRITE|_S_IEXEC)
#endif /* __MINGW32__ */
// (signal?)
#define SIGPIPE	_S_IFIFO
// (directory)
#define __S_ISTYPE(mode, mask)  (((mode) & _S_IFMT) == (mask))
#ifndef __MINGW32__
#define S_ISFIFO(mode)	__S_ISTYPE((mode), _S_IFIFO)
#define S_ISDIR(mode)	__S_ISTYPE((mode), _S_IFDIR)
#define S_ISREG(mode)	__S_ISTYPE((mode), _S_IFREG)
#endif /* __MINGW32__ */

// * functions *
// *** str ***
int strcasecmp( const char *s1, const char *s2 );

int strncasecmp( const char *s1, const char *s2, size_t n );

// *** dir ***
//#include <windows.h>
//#include <io.h>
#ifndef __MINGW32__
typedef void * HANDLE;
#endif /* __MINGW32__ */

#ifndef _FINDDATA_T_DEFINED
#ifndef __MINGW32__
typedef unsigned long _fsize_t; /* Could be 64 bits for Win32 */
#endif /* __MINGW32__ */
struct _finddata_t {
    unsigned    attrib;
    time_t      time_create;    /* -1 for FAT file systems */
    time_t      time_access;    /* -1 for FAT file systems */
    time_t      time_write;
    _fsize_t    size;
    char        name[260];
};
#endif

struct dirent {
	long d_ino;
	unsigned short d_reclen;
	unsigned short d_namlen;
	char *d_name;
};
typedef struct {
	struct _finddata_t	dd_dta;
	struct dirent		dd_dir;
	long			dd_handle;
	short			dd_stat;
	char			dd_name[1];
	HANDLE hFind;
	char *dirname;
} DIR;

DIR *opendir( const char *name );
int closedir( DIR *dir );
struct dirent *readdir( DIR *dir );

#ifdef __MINGW32__
struct timezone {
  int tz_minuteswest;
  int tz_dsttime;
};
#endif

// *** stat ***
int lstat( const char *file_name, struct stat *buf );

// *** sys/wait ***
pid_t waitpid( pid_t pid, int *status, int options );

// *** sys/time ***
int gettimeofday( struct timeval *tv, struct timezone *tz );

// *** unistd ***
uid_t getuid( void );
int setpgid( pid_t pid, pid_t pgid );
pid_t getppid( void );
pid_t fork( void );
unsigned int sleep( unsigned int seconds );
int link( const char *oldpath, const char *newpath );
int usleep( unsigned long usec );

// *** stdlib ***
long int random( void );
void srandom( unsigned int seed );
int truncate( const char *path, off_t length );

// *** signal ***
int kill( pid_t pid, int sig );

// *** stdio ***
FILE *popen( const char *command, const char *type );
int pclose( FILE *stream );

// *** misc ***
gchar *read_w32_registry_string( char *parent, char *section, char *key );
gchar *get_content_type_from_registry_with_ext( char *ext );

#endif
